# -*- coding: utf-8 -*-
""" PyMiniRacer main wrappers """
# pylint: disable=bad-whitespace,too-few-public-methods

import sys
import os
import json
import ctypes
import threading
import datetime
import fnmatch

from pkg_resources import resource_listdir, resource_filename

# In python 3 the extension file name depends on the python version
try:
    EXTENSION_NAME = fnmatch.filter(resource_listdir('py_mini_racer', '.'), '_v8*.so')[0]
    EXTENSION_PATH = resource_filename('py_mini_racer', EXTENSION_NAME)
except NotImplementedError:
    if not hasattr(sys, "_MEIPASS"):
        raise
    __location__ = os.path.join(sys._MEIPASS, "_v8") # pylint: disable=no-member, protected-access
    EXTENSION_NAME = fnmatch.filter(os.listdir(__location__), '_v8*.so')[0]
    EXTENSION_PATH = os.path.join(__location__, EXTENSION_NAME)

class MiniRacerBaseException(Exception):
    """ base MiniRacer exception class """
    pass

class JSParseException(MiniRacerBaseException):
    """ JS could not be parsed """
    pass

class JSEvalException(MiniRacerBaseException):
    """ JS could not be executed """
    pass

class JSConversionException(MiniRacerBaseException):
    """ type could not be converted """
    pass

class WrongReturnTypeException(MiniRacerBaseException):
    """ type returned by JS cannot be parsed """
    pass

class JSFunction(object):
    """ type for JS functions """
    pass


def is_unicode(value):
    """ Check if a value is a valid unicode string, compatible with python 2 and python 3

    >>> is_unicode(u'foo')
    True
    >>> is_unicode(u'✌')
    True
    >>> is_unicode(b'foo')
    False
    >>> is_unicode(42)
    False
    >>> is_unicode(('abc',))
    False
    """
    python_version = sys.version_info[0]

    if python_version == 2:
        return isinstance(value, unicode)
    elif python_version == 3:
        return isinstance(value, str)
    else:
        raise NotImplementedError()


class MiniRacer(object):
    """ Ctypes wrapper arround binary mini racer
        https://docs.python.org/2/library/ctypes.html
    """

    def __init__(self):
        """ Init a JS context """

        self.ext = ctypes.CDLL(EXTENSION_PATH)

        self.ext.mr_init_context.restype = ctypes.c_void_p
        self.ext.mr_eval_context.argtypes = [
            ctypes.c_void_p,
            ctypes.c_char_p,
            ctypes.c_int]
        self.ext.mr_eval_context.restype = ctypes.POINTER(PythonValue)

        self.ext.mr_free_value.argtypes = [ctypes.c_void_p]

        self.ext.mr_free_context.argtypes = [ctypes.c_void_p]

        self.ext.mr_heap_stats.argtypes = [ctypes.c_void_p]
        self.ext.mr_heap_stats.restype = ctypes.POINTER(PythonValue)

        self.ctx = self.ext.mr_init_context()

        self.lock = threading.Lock()

    def free(self, res):
        """ Free value returned by mr_eval_context """

        self.ext.mr_free_value(res)

    def execute(self, js_str):
        """ Exec the given JS value """

        wrapped = "(function(){return (%s)})()" % js_str
        return self.eval(wrapped)

    def eval(self, js_str):
        """ Eval the JavaScript string """

        if is_unicode(js_str):
            bytes_val = js_str.encode("utf8")
        else:
            bytes_val = js_str

        self.lock.acquire()
        res = self.ext.mr_eval_context(self.ctx, bytes_val, len(bytes_val))
        self.lock.release()

        if bool(res) is False:
            raise JSConversionException()
        python_value = res.contents.to_python()
        self.free(res)
        return python_value

    def call(self, identifier, *args, **kwargs):
        """ Call the named function with provided arguments
        You can pass a custom JSON encoder by passing it in the encoder
        keyword only argument.
        """

        encoder = kwargs.get('encoder', None)

        json_args = json.dumps(args, separators=(',', ':'), cls=encoder)
        js = "{identifier}.apply(this, {json_args})"
        return self.eval(js.format(identifier=identifier, json_args=json_args))

    def heap_stats(self):
        """ Return heap statistics """

        self.lock.acquire()
        res = self.ext.mr_heap_stats(self.ctx)
        self.lock.release()

        python_value = res.contents.to_python()
        self.free(res)
        return python_value

    def __del__(self):
        """ Free the context """

        self.ext.mr_free_context(self.ctx)


class PythonTypes(object):
    """ Python types identifier - need to be coherent with
    mini_racer_extension.cc """

    null      =   1
    bool      =   2
    integer   =   3
    double    =   4
    str_utf8  =   5
    array     =   6
    hash      =   7
    date      =   8

    function  = 100

    execute_exception = 200
    parse_exception = 201

    invalid   = 300


class PythonValue(ctypes.Structure):
    """ Map to C PythonValue """
    _fields_ = [("value", ctypes.c_void_p),
                ("type", ctypes.c_int),
                ("len", ctypes.c_size_t)]

    def __str__(self):
        return str(self.to_python())

    def _double_value(self):
            ptr = ctypes.c_char_p.from_buffer(self)
            return ctypes.c_double.from_buffer(ptr).value

    def to_python(self):
        """ Return an object as native Python """

        result = None
        if self.type == PythonTypes.null:
            result = None
        elif self.type == PythonTypes.bool:
            result = self.value == 1
        elif self.type == PythonTypes.integer:
            if self.value is None:
                result = 0
            else:
                result = ctypes.c_int32(self.value).value
        elif self.type == PythonTypes.double:
            result = self._double_value()
        elif self.type == PythonTypes.str_utf8:
            buf = ctypes.c_char_p(self.value)
            ptr = ctypes.cast(buf, ctypes.POINTER(ctypes.c_char))
            result = ptr[0:self.len].decode("utf8")
        elif self.type == PythonTypes.array:
            if self.len == 0:
                return []
            ary = []
            ary_addr = ctypes.c_void_p.from_address(self.value)
            ptr_to_ary = ctypes.pointer(ary_addr)
            for i in range(self.len):
                pval = PythonValue.from_address(ptr_to_ary[i])
                ary.append(pval.to_python())
            result = ary
        elif self.type == PythonTypes.hash:
            if self.len == 0:
                return {}
            res = {}
            hash_ary_addr = ctypes.c_void_p.from_address(self.value)
            ptr_to_hash = ctypes.pointer(hash_ary_addr)
            for i in range(self.len):
                pkey = PythonValue.from_address(ptr_to_hash[i*2])
                pval = PythonValue.from_address(ptr_to_hash[i*2+1])
                res[pkey.to_python()] = pval.to_python()
            result = res
        elif self.type == PythonTypes.function:
            result = JSFunction()
        elif self.type == PythonTypes.parse_exception:
            msg = ctypes.c_char_p(self.value).value
            raise JSParseException(msg)
        elif self.type == PythonTypes.execute_exception:
            msg = ctypes.c_char_p(self.value).value
            raise JSEvalException(msg.decode('utf-8', errors='replace'))
        elif self.type == PythonTypes.date:
            timestamp = self._double_value()
            # JS timestamp are milliseconds, in python we are in seconds
            result = datetime.datetime.utcfromtimestamp(timestamp / 1000.)
        else:
            raise WrongReturnTypeException("unknown type %d" % self.type)
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Basic JS types tests """

import unittest
import json
import time

from datetime import datetime

from py_mini_racer import py_mini_racer


class Test(unittest.TestCase):
    """ Test basic types """

    def valid(self, py_val, **kwargs):
        if 'testee' in kwargs:
            testee = kwargs['testee']
        else:
            testee = py_val
        js_str = json.dumps(py_val)
        parsed = self.mr.execute(js_str)
        self.assertEqual(testee, parsed)

    def setUp(self):

        self.mr = py_mini_racer.MiniRacer()

    def test_str(self):
        self.valid("'a string'")
        self.valid("'a ' + 'string'")

    def test_unicode(self):
        ustr = u"\N{GREEK CAPITAL LETTER DELTA}"
        res = self.mr.eval("'" + ustr + "'")
        self.assertEqual(ustr, res)

    def test_numbers(self):
        self.valid(1)
        self.valid(1.0)
        self.valid(2**16)
        self.valid(2**31-1)
        self.valid(2**31)
        self.valid(2**33)

    def test_arrays(self):
        self.valid([1])
        self.valid([])
        self.valid([1, 2, 3])
        # Nested
        self.valid([1, 2, ['a', 1]])

    def test_none(self):
        self.valid(None)

    def test_hash(self):
        self.valid({})
        self.valid('{}')
        self.valid({'a': 1})
        self.valid({" ": {'z': 'www'}})

    def test_complex(self):

        self.valid({
            1: [
                1, 2, 'qwe', {
                    'z': [
                        4, 5, 6, {
                            'eqewr': 1,
                            'zxczxc': 'qweqwe',
                            'z': {1: 2}
                        }
                    ]
                }
            ], 'qwe': 1
        })

    def test_function(self):
        res = self.mr.eval('var a = function(){}; a')
        self.assertTrue(isinstance(res, py_mini_racer.JSFunction))

    def test_invalid_key(self):

        fun = """
            var o = {};
            o.__defineGetter__("bar", function() { return null(); });
            o
        """
        with self.assertRaises(py_mini_racer.JSConversionException):
            self.mr.eval(fun)

    def test_timestamp(self):
        val = int(time.time())
        res = self.mr.eval("var a = new Date(%d); a" % (val * 1000))
        self.assertEqual(res, datetime.utcfromtimestamp(val))

    def test_date(self):
        res = self.mr.eval("var a = new Date(Date.UTC(2014, 0, 2, 3, 4, 5)); a")
        self.assertEqual(res, datetime(2014, 1, 2, 3, 4, 5))

    def test_exception(self):
        js_source = """
        var f = function(arg) {
            throw 'error: '+arg
            return nil
        }"""

        self.mr.eval(js_source)

        with self.assertRaises(py_mini_racer.JSEvalException) as cm:
            self.mr.eval("f(42)")

        self.assertIn('error: 42', cm.exception.args[0])


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Basic JS types tests """

import unittest

from py_mini_racer import py_mini_racer

class Test(unittest.TestCase):
    """ Test basic types """

    def setUp(self):

        self.mr = py_mini_racer.MiniRacer()

    def test_invalid(self):

        with self.assertRaises(py_mini_racer.JSEvalException):
            self.mr.eval("invalid")

    def test_global(self):
        self.mr.eval('var xabc = 22;')
        self.assertEqual(22, self.mr.eval('xabc'))


    def test_fun(self):
        res = self.mr.eval('var x = function(y) {return y+1;}')

        self.assertEqual(2,   self.mr.eval('x(1)'))
        self.assertEqual(11,  self.mr.eval('x(10)'))
        self.assertEqual(101, self.mr.eval('x(100)'))

    def test_multiple_ctx(self):

        c1 = py_mini_racer.MiniRacer()
        c2 = py_mini_racer.MiniRacer()
        c3 = py_mini_racer.MiniRacer()

        c1.eval('var x = 1')
        c2.eval('var x = 2')
        c3.eval('var x = 3')
        self.assertEqual(c1.eval('(x)'), 1)
        self.assertEqual(c2.eval('(x)'), 2)
        self.assertEqual(c3.eval('(x)'), 3)

    def test_exception_thrown(self):
        context = py_mini_racer.MiniRacer()

        js_source = "var f = function() {throw 'error'};"

        context.eval(js_source)

        with self.assertRaises(py_mini_racer.JSEvalException):
            context.eval("f()")

    def test_cannot_parse(self):

        context = py_mini_racer.MiniRacer()

        js_source = "var f = function("

        with self.assertRaises(py_mini_racer.JSParseException):
            context.eval(js_source)

    def test_null_byte(self):

        context = py_mini_racer.MiniRacer()

        s = "\x00 my string!"

        # Try return a string including a null byte
        in_val = "var str = \"" + s + "\"; str;"
        result = context.eval(in_val)
        self.assertEqual(result, s)


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Basic JS types tests """

from io import open

import unittest
import os

from py_mini_racer import py_mini_racer


class Test(unittest.TestCase):
    """ Test basic types """

    def test_babel(self):

        context = py_mini_racer.MiniRacer()

        path = os.path.join(os.path.dirname(__file__), 'fixtures/babel.js')
        babel_source = open(path, "r", encoding='utf-8').read()
        source = """
          var self = this;
          %s
          babel.eval = function(code) {
            return eval(babel.transform(code)["code"]);
          }
        """ % babel_source
        context.eval(source)
        self.assertEqual(64, context.call("babel.eval", "((x) => x * x)(8)"))


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
import logging
import os
import os.path
import subprocess
import multiprocessing

from glob import glob
from os.path import join, dirname, abspath
from contextlib import contextmanager


logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

V8_VERSION = "5.1.281.67"


def local_path(path):
    """ Return path relative to this file
    """
    current_path = dirname(__file__)
    return abspath(join(current_path, path))


VENDOR_PATH = local_path('../../vendor/depot_tools')
PATCHES_PATH = local_path('../../patches')


def call(cmd):
    LOGGER.debug("Calling: '%s' from working directory %s", cmd, os.getcwd())
    current_env = os.environ
    depot_tools_env = '{}:{}'.format(VENDOR_PATH, os.environ['PATH'])
    current_env['PATH'] = depot_tools_env
    return subprocess.check_call(cmd, shell=True, env=current_env)


@contextmanager
def chdir(new_path, make=False):
    old_path = os.getcwd()

    if make is True:
        try:
            os.mkdir(new_path)
        except OSError:
            pass

    try:
        yield os.chdir(new_path)
    finally:
        os.chdir(old_path)


def ensure_v8_src():
    """ Ensure that v8 src are presents and up-to-date
    """
    path = local_path('v8')

    if not os.path.isdir(path):
        fetch_v8(path)
    else:
        update_v8(path)

    checkout_v8_version(local_path("v8/v8"), V8_VERSION)
    dependencies_sync(path)


def fetch_v8(path):
    """ Fetch v8
    """
    with chdir(abspath(path), make=True):
        call("fetch v8")


def update_v8(path):
    """ Update v8 repository
    """
    with chdir(path):
        call("gclient fetch")


def checkout_v8_version(path, version):
    """ Ensure that we have the right version
    """
    with chdir(path):
        call("git checkout {} -- .".format(version))


def dependencies_sync(path):
    """ Sync v8 build dependencies
    """
    with chdir(path):
        call("gclient sync")


def gyp_defines():
    defines = []

    # Do not use an external snapshot as we don't really care for binary size
    # and so we don't need to ship the blobs
    defines.append('v8_use_external_startup_data=0')

    # Do not use the GPLv3 ld.gold binary on Linux
    defines.append('linux_use_bundled_gold=0')

    return 'GYP_DEFINES="{}"'.format(" ".join(defines))


def make_flags():
    """ Returns compilation flags
    """
    flags = []

    # Activate parallel build process
    flags.append('-j {:d}'.format(multiprocessing.cpu_count()))

    # Disable i18n
    flags.append('i18nsupport=off')

    # Disable werror as this version of v8 is getting difficult to maintain
    # with it on
    flags.append('werror=no')

    flags.append(gyp_defines())

    return ' '.join(flags)


def make(path, flags, target='native'):
    """ Create a release of v8
    """
    with chdir(path):
        call("make {} {}".format(target, flags))

def patch_v8():
    """ Apply patch on v8
    """
    path = local_path('v8/v8')
    patches_paths = PATCHES_PATH
    apply_patches(path, patches_paths)


def apply_patches(path, patches_path):
    with chdir(path):

        if not os.path.isfile('.applied_patches'):
            open('.applied_patches', 'w').close()

        with open('.applied_patches', 'r+') as applied_patches_file:
            applied_patches = set(applied_patches_file.read().splitlines())

            for patch in glob(join(patches_path, '*.patch')):
                if patch not in applied_patches:
                    call("patch -p1 -N < {}".format(patch))

                    applied_patches_file.write(patch + "\n")


def build_v8(target):
    ensure_v8_src()
    patch_v8()
    flags = make_flags()
    make(local_path('v8/v8'), flags, target)


if __name__ == '__main__':
    build_v8('native')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" Basic JS call functions """

import json
import unittest

from datetime import datetime

from py_mini_racer import py_mini_racer


class TestEval(unittest.TestCase):
    """ Test calling a function """

    def setUp(self):

        self.mr = py_mini_racer.MiniRacer()

    def test_call_js(self):

        js_func = """var f = function(args) {
            return args.length;
        }"""

        self.mr.eval(js_func)

        self.assertEqual(self.mr.call('f', list(range(5))), 5)
        self.assertEqual(self.mr.call('f', list(range(10))), 10)
        self.assertEqual(self.mr.call('f', list(range(20))), 20)

    def test_call_custom_encoder(self):

        # Custom encoder for dates
        class CustomEncoder(json.JSONEncoder):

            def default(self, obj):
                if isinstance(obj, datetime):
                    return obj.isoformat()

                return json.JSONEncoder.default(self, obj)

        now = datetime.now()
        js_func = """var f = function(args) {
            return args;
        }"""
        self.mr.eval(js_func)

        self.assertEqual(self.mr.call('f', now, encoder=CustomEncoder),
                         now.isoformat())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import pip
import sys
import codecs
import pkg_resources

from subprocess import check_call
from os.path import dirname, abspath, join, isfile, isdir, basename

from distutils.file_util import copy_file

try:
    from setuptools import setup, Extension, Command
    from setuptools.command.build_ext import build_ext
    from setuptools.command.install import install
except ImportError:
    from distutils.core import setup, Extension, Command
    from distutils.command.build_ext import build_ext
    from distutils.command.install import install

import py_mini_racer
from py_mini_racer.extension.v8_build import build_v8

V8_PATH = os.environ.get("PY_MINI_RACER_V8_PATH")

with codecs.open('README.rst', 'r', encoding='utf8') as readme_file:
    readme = readme_file.read()

    # Convert local image links by their github equivalent
    readme = readme.replace(".. image:: data/",
                            ".. image:: https://github.com/sqreen/PyMiniRacer/raw/master/data/")

with codecs.open('HISTORY.rst', 'r', encoding='utf8') as history_file:
    history = history_file.read().replace('.. :changelog:', '')


def _parse_requirements(filepath):
    pip_version = list(map(int, pkg_resources.get_distribution('pip').version.split('.')[:2]))
    if pip_version >= [10, 0]:
        from pip._internal.download import PipSession
        from pip._internal.req import parse_requirements
        raw = parse_requirements(filepath, session=PipSession())
    elif pip_version >= [6, 0]:
        from pip.download import PipSession
        from pip.req import parse_requirements
        raw = parse_requirements(filepath, session=PipSession())
    else:
        from pip.req import parse_requirements
        raw = parse_requirements(filepath)

    return [str(i.req) for i in raw]


requirements = _parse_requirements('requirements/prod.txt')
setup_requires = _parse_requirements('requirements/setup.txt')
test_requirements = _parse_requirements('requirements/test.txt')


def get_target():
    return os.environ.get('V8_TARGET', 'native')


def local_path(path):
    """ Return path relative to this file
    """
    current_path = dirname(__file__)
    return abspath(join(current_path, path))


V8_LIB_DIRECTORY = local_path('py_mini_racer/extension/v8/v8')
V8_STATIC_LIBRARIES = ['libv8_base.a', 'libv8_libbase.a', 'libv8_libplatform.a',
                       'libv8_nosnapshot.a']


def is_v8_built():
    """ Check if v8 has been built
    """
    if V8_PATH:
        return True
    return all(isfile(static_filepath) for static_filepath in get_raw_static_lib_path())


def is_depot_tools_checkout():
    """ Check if the depot tools submodule has been checkouted
    """
    return isdir(local_path('vendor/depot_tools'))


def libv8_object(object_name):
    """ Return a path for object_name which is OS independent
    """
    filename = join(V8_LIB_DIRECTORY, "out/{}/".format(get_target()),
                    object_name)
    if not isfile(filename):
        filename = join(V8_LIB_DIRECTORY,
                        "out/{}/obj.target/tools/gyp/".format(get_target()),
                        object_name)

    if not isfile(filename):
        filename = join(V8_LIB_DIRECTORY,
                        "out/{}/".format(get_target()),
                        object_name)

    return filename


def get_raw_static_lib_path():
    """ Return the list of the static libraries files ONLY, use
    get_static_lib_paths to get the right compilation flags
    """
    return [libv8_object(static_file) for static_file in V8_STATIC_LIBRARIES]


def get_static_lib_paths():
    """ Return the required static libraries path
    """
    libs = []
    is_linux = sys.platform.startswith('linux')
    if is_linux:
        libs += ['-Wl,--start-group']
    libs += get_raw_static_lib_path()
    if is_linux:
        libs += ['-Wl,--end-group']
    return libs

EXTRA_LINK_ARGS = [
    '-ldl',
    '-fstack-protector',
]


# Per platform customizations
if sys.platform[:6] == "darwin":
    EXTRA_LINK_ARGS += ['-stdlib=libstdc++', '-lpthread']
elif sys.platform.startswith('linux'):
    EXTRA_LINK_ARGS += ['-lrt']


PY_MINI_RACER_EXTENSION = Extension(
    name="py_mini_racer._v8",
    sources=['py_mini_racer/extension/mini_racer_extension.cc'],
    include_dirs=[V8_LIB_DIRECTORY, join(V8_LIB_DIRECTORY, 'include')],
    extra_objects=get_static_lib_paths(),
    extra_compile_args=['-std=c++0x', '-rdynamic', '-fpermissive', '-fno-common'],
    extra_link_args=EXTRA_LINK_ARGS
)


class MiniRacerBuildExt(build_ext):

    def build_extension(self, ext):
        """ Compile manually the py_mini_racer extension, bypass setuptools
        """
        try:
            if not is_v8_built():
                self.run_command('build_v8')

            self.debug = True
            if V8_PATH:
                dest_filename = join(self.build_lib, "py_mini_racer")
                copy_file(V8_PATH, dest_filename, verbose=self.verbose, dry_run=self.dry_run)
            else:
                build_ext.build_extension(self, ext)

        except Exception as e:
            # Alter message
            err_msg = """py_mini_racer failed to build, ensure you have an up-to-date pip (>= 8.1) to use the wheel instead
            To update pip: 'pip install -U pip'
            See also: https://github.com/sqreen/PyMiniRacer#binary-builds-availability

            Original error: %s"""

            raise Exception(err_msg % repr(e))


class MiniRacerBuildV8(Command):

    description = 'Compile vendored v8'
    user_options = [
    ]

    def initialize_options(self):
        """Set default values for options."""

    def finalize_options(self):
        """Post-process options."""
        pass

    def run(self):
        if V8_PATH:
            return

        if not is_depot_tools_checkout():
            print("cloning depot tools submodule")
            # Clone the depot_tools repository, easier than using submodules
            check_call(['git', 'init'])
            check_call(['git', 'clone', 'https://chromium.googlesource.com/chromium/tools/depot_tools.git', 'vendor/depot_tools'])

        if not is_v8_built():
            print("building v8")
            target = os.environ.get('V8_TARGET', 'native')
            build_v8(target)
        else:
            print("v8 is already built")

setup(
    name='py_mini_racer',
    version=py_mini_racer.__version__,
    description="Minimal, modern embedded V8 for Python.",
    long_description=readme + '\n\n' + history,
    author='Sqreen',
    author_email='hey@sqreen.io',
    url='https://github.com/sqreen/PyMiniRacer',
    packages=[
        'py_mini_racer',
        'py_mini_racer.extension'
    ],
    ext_modules=[PY_MINI_RACER_EXTENSION],
    package_dir={'py_mini_racer':
                 'py_mini_racer'},
    include_package_data=True,
    setup_requires=setup_requires,
    install_requires=requirements,
    license="ISCL",
    zip_safe=False,
    keywords='py_mini_racer',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    cmdclass={
        'build_ext': MiniRacerBuildExt,
        'build_v8': MiniRacerBuildV8,
    }
)
